use std::collections::HashMap;

use statement::{ConstVar, Enum, Function, StaticVar, Struct};

use crate::{common::loc::LocBox, validator::predefined::TypeType};
pub use crate::tokeniser::tokentype::*;

pub mod expr;
pub mod literal;
pub mod statement;
pub mod typ;

#[derive(Debug, Clone)]
pub struct Program {
    pub ast: expr::Block,
    pub structs: HashMap<Ident, LocBox<Struct>>,
    pub enums: HashMap<Ident, LocBox<Enum>>,
    pub types: HashMap<Ident, TypeType>,
    pub functions: HashMap<Ident, LocBox<Function>>,
    pub member_functions: HashMap<Ident, HashMap<Ident, LocBox<Function>>>,
    pub static_vars: HashMap<Ident, StaticVar>,
    pub const_vars: HashMap<Ident, ConstVar>
} 

#[derive(Debug, Clone)]
pub enum Ast {
    Expr(LocBox<expr::Expr>),
    Statement(LocBox<statement::Statement>),
}


